//
// Module: Multiply Add (MACC) Element
//
// Description: Verilog instantiation template for 
// DSP48 embedded MAC blocks arranged as a Systolic
// 23 tap FIR filter. 23 slices used, 1 slice per tap.
// 
//
// Device: Virtex-4 Family
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////


module MACC (A, B, CLK, LOAD, CE, RST, DOUT);
							 
input             CLK, CE, RST, LOAD;
input   [17:0]    A, B;

output  [47:0]   	DOUT;

wire    [6:0]     OPMODE;

reg  [17:0]		  COEF; 


											  
//          
// XtremeDSP Slice 
//

DSP48 MACC0
  (
    .A(A),                 // Input A to Multiplier
    .B(B),                 // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(18'b0),          //
    .PCIN(48'b0),          //
    .OPMODE(OPMODE),       //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),              //
    .CEB(CE),              //
    .CEC(CE),              //
    .CEP(CE),              //
    .CEM(CE),              //
    .CECTRL(CE),           //
    .CECARRYIN(CE),        //
	 .CECINSUB(CE),         //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),              //
    .P(DOUT),              //
    .PCOUT()               //
  );

//synthesis attribute AREG of MADD0 is "1";
//synthesis attribute BREG of MADD0 is "1";
//synthesis attribute CARRYINREG of MADD0 is "1";
//synthesis attribute MREG of MADD0 is "1";
//synthesis attribute PREG of MADD0 is "1";
//synthesis attribute OPMODEREG of MADD0 is "1";
//synthesis attribute SUBTRACTREG of MADD0 is "1";
//synthesis attribute CARRYINSELREG of MADD0 is "1";
//synthesis attribute B_INPUT of MADD0 is "DIRECT";

//synthesis translate_off 
defparam MADD0.AREG = 2'b01;
defparam MADD0.BREG = 2'b01;
defparam MADD0.CARRYINREG = 1'b1;
defparam MADD0.MREG = 1'b1;
defparam MADD0.PREG = 1'b1;
defparam MADD0.OPMODEREG = 1'b1;
defparam MADD0.SUBTRACTREG = 1'b1;
defparam MADD0.CARRYINSELREG = 1'b1;
defparam MADD0.B_INPUT = "DIRECT";
//synthesis translate_on

assign OPMODE[5] = ~(LOAD);
assign OPMODE[4:0] = 5'b00101;
assign OPMODE[6] = 1'b0;

endmodule

module DSP48 (BCOUT, P, PCOUT, A, B, C, OPMODE, SUBTRACT, CARRYIN, CECINSUB, CARRYINSEL,
              BCIN, PCIN, CLK, CEA, CEB, CEC, CEP, CEM, CECTRL, CECARRYIN,
			  RSTA, RSTB, RSTC, RSTP, RSTM, RSTCTRL, RSTCARRYIN) /* synthesis syn_black_box */;
		output [47:0] P, PCOUT;
		output [17:0] BCOUT;

		input [17:0] A, B, BCIN;
		input [47:0] PCIN,C;

		input [6:0] OPMODE;
		input [1:0] CARRYINSEL;
		input CLK, CEA, CEB, CEC, CEP, CEM, CECTRL, SUBTRACT, CARRYIN, CECARRYIN, CECINSUB;
		input RSTA, RSTB, RSTC, RSTP, RSTM, RSTCTRL, RSTCARRYIN;

endmodule

