//
// Module: Control
//
// Description:  Control Logic for MACC FIR filter
// 
//
// Device: Virtex-4 Family
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////


module control (CLK, CE, ADDR_A, WE_A, ADDR_B, RST);
							 
input             CLK, CE, RST;

output			  	WE_A;
output  [6:0]    	ADDR_A, ADDR_B;

wire              CONTROL;
wire    [6:0]     OPMODE;

					  
///////////////////////////////////////////////////////////////////
//
//	  CREATE THE COUNTERS AND WE PULSE
//
///////////////////////////////////////////////////////////////////

    reg [1:0] ADDR_COUNT;
   
    always @(posedge CLK)
	 begin
      if (RST)
         ADDR_COUNT <= 0;
		
		else begin
			if (CE) begin
             if (ADDR_COUNT == 2*96-1)	begin
				 	  ADDR_COUNT <= 96;
				 end
				 else begin
					  ADDR_COUNT <= ADDR_COUNT + 1;
				 end
			end
	   end
	 end

	 reg [1:0] DATA_COUNT;
   
    always @(posedge CLK)
	 begin
      if (RST)
         DATA_COUNT <= 0;
		
		else begin
			if (~(CONTROL)) begin
             if (DATA_COUNT == 96-1)	begin
				 	  DATA_COUNT <= 0;
				 end
				 else begin
					  DATA_COUNT <= DATA_COUNT + 1;
				 end
			end
	   end
	 end

 	assign CONTROL = (ADDR_COUNT == 2*96-1) ? 1 : 0; 

   reg WE_REG ; 

	always @(posedge CLK)
	begin
       if(CE)
		     WE_REG <= CONTROL;
	end

	assign WE_A = WE_REG;
 	assign ADDR_A = ADDR_COUNT;
	assign ADDR_B = ADDR_COUNT;

endmodule	


 