--
-- Entity: Control
--
-- Description: 
--
-- VHDL Reference Design for Single Multiplier MAC FIR Filter using the DSP48
-- Control section
--
--
--
-- Device: Virtex-4 Family
--
-- Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
--


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;

use work.filter_param.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity control is
  port (
    clk: in std_logic;
    ce: in std_logic;
    addr_A: out integer range 0 to (2*N-1);
    we_A: out std_logic;
    addr_B: out integer range N to (2*N-1);
    rst: in std_logic
  );
end control;

architecture behavioral of control is
  
  
  signal data_count : integer range 0 to (2*N-1);
  signal addr_count : integer range N to (2*N-1);
  signal data_count_sig, addr_count_sig : std_logic_vector (a-1 downto 0);  
  signal data_count_ce : std_logic; 
  signal addr_limit, data_limit : std_logic;


--Custom instantiated constant comparator as synthesis does not do a good job of inferring it
component compare is
    
	 generic (
	 
	 INIT1 : bit_vector(15 downto 0):="1011101110101010";
	 INIT2 : bit_vector(15 downto 0):="1011101110101010"
	 );
	 port (
	 	a: in std_logic_vector(6 downto 0);
		clk : in std_logic;
		equal : out std_logic
		);

end component;
 
 
begin
  
  -- Create the data counter
  process (clk, ce, rst, data_count_ce)
  begin
  	
  	if ( clk'event and clk='1' ) then
	 if (rst = '1')	then
      data_count <= 0;
	 
  	 else
	 
  	      if (data_count_ce = '1') then
				if (data_limit = '1') then
	             data_count <= 0;
				
				else
				    data_count <= data_count + 1;
            
				end if;  	      
  	      end if;
  	      
		
	end if;
  end if;
  end process ;
  
  -- Create the address counter
  process (clk, ce, rst)
  begin

  if ( clk'event and clk='1' ) then
	 if (rst = '1')	then
  	      addr_count <= N;
	 
  	 else
	      
			if (ce ='1') then
			   if (addr_limit = '1') then
	            addr_count <= N;
  	         
			   else
			      addr_count <= addr_count + 1;

			   end if;
  	      
			end if;
			
		 --else
			--	addr_count <= addr_count;
		 		  
	    --end if;
	end if;
  end if;
  end process ;


  data_count_ce <= not(addr_limit);

  addr_count_sig <= conv_std_logic_vector(addr_count, a);
  data_count_sig <= conv_std_logic_vector(data_count, a);
  
  -- Create the WE signal

  --control <= '1' when addr_count = 2*N-2 else 
  --           '0';
  


   C2 : compare 
	 generic map(
	   init1 => "0100000000000000",
		init2 => "0000000000001000"
	 )
	 
	 port map (
	 	a =>  addr_count_sig,
		clk => clk,
		equal => addr_limit
		);

  -- THIS IS PUT INTO THE CONSTANT INSTANTIATED COMPARE
  --data_limit  <= '1' when data_count = N-2 else 
  --           '0';
  
	 C1 : compare 
	 generic map(
	   init1 => "0100000000000000",
		init2 => "0000000000000010"
	 )
	 
	 port map (
	 	a => data_count_sig,
		clk => clk,
		equal => data_limit
		);

  -- Delay the WE signal
  process (clk, ce, rst)
    begin
    
     if ( clk'event and clk='1' ) then
    
	 		if (rst = '1') then
    	
    	   we_A <= '0';
			
			else	
    	   if (ce = '1') then      
    	      we_A <= addr_limit;
    	      
  	      end if;
			end if;
    	end if;
  end process ;
  
	-- Output the Count signals
	addr_A <= data_count;
	addr_B <= addr_count;
  
end behavioral;